<?php


//======================================
//     Server settings
//======================================
$SERVER_URL="https://127.0.0.1:12345/api";
$API_KEY="1EC2-FCD9";



//======================================
//     Helper functions
//======================================

// Create curl connection with HTTPS options
function createCurlConnection($api_call) {
	global $SERVER_URL;

	$ch = curl_init($SERVER_URL.$api_call);
	
	//for debugging
	//$fp = fopen("example_homepage.txt", "w");
	//curl_setopt($ch, CURLOPT_FILE, $fp);
	
	curl_setopt($ch, CURLOPT_HEADER, 0);
	curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
	curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 2);
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
	
	return $ch;
}

// Generate client UUID, we use it for a client nonce (cnonce)
function gen_uuid() {
    return sprintf( '%04x%04x-%04x-%04x-%04x-%04x%04x%04x',
        // 32 bits for "time_low"
        mt_rand( 0, 0xffff ), mt_rand( 0, 0xffff ),

        // 16 bits for "time_mid"
        mt_rand( 0, 0xffff ),

        // 16 bits for "time_hi_and_version",
        // four most significant bits holds version number 4
        mt_rand( 0, 0x0fff ) | 0x4000,

        // 16 bits, 8 bits for "clk_seq_hi_res",
        // 8 bits for "clk_seq_low",
        // two most significant bits holds zero and one for variant DCE1.1
        mt_rand( 0, 0x3fff ) | 0x8000,

        // 48 bits for "node"
        mt_rand( 0, 0xffff ), mt_rand( 0, 0xffff ), mt_rand( 0, 0xffff )
    );
}


//======================================
//     Authentication functions
//======================================

// nonce is valid for 30 seconds, use it to login within those 30 seconds
function getNonce() {
	
	$ch = createCurlConnection("/auth/nonce");
		
	$body = curl_exec($ch);
	$code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
	$result = null;
	
	if ($code==200) {
		$result = json_decode($body, true);
		$result = $result["authNonce"];
	}
	curl_close($ch);
	//fclose($fp);
	
	return $result;
}

// SessionID received will be invalidated
// when the session is not being used for 5 minutes
// re-request when receiving HTTP 403, do not store sessionid
function login($nonce, $cnonce) {
	
	global $API_KEY;

	$hash_source = $nonce.":".$cnonce.":".$API_KEY;
	$sha256 = hash('sha256', $hash_source, true);
	$hash = base64_encode($sha256);

	$ch = createCurlConnection("/auth/login?nonce=".urlencode($nonce)."&cnonce=".urlencode($cnonce)."&hash=".urlencode($hash));
		
	$body = curl_exec($ch);
	$code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
	$result = null;
	
	if ($code==200) {
		$result = json_decode($body, true);
		$result=$result["sessionID"];
	}
	curl_close($ch);
	
	return $result;
}

//======================================
//     Example API functions
//======================================

//Adding a NZB via a URL
function addNZB($url, $sessionID) {

	$ch = createCurlConnection("/nzb/add?url=".urlencode($url)."&sessionid=".urlencode($sessionID));
		
	$body = curl_exec($ch);
	$code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
	$result = null;
	
	echo "HTTP result code: ".$code."<br>\n";
	if ($code==200) {
		$result = json_decode($body, true);
		echo $body;
	}
	curl_close($ch);
	
	return $result;
}

//List NZBs from queue
function listNZBs($sessionID) {
	$ch = createCurlConnection("/nzb?sessionid=".urlencode($sessionID));
		
	$body = curl_exec($ch);
	$code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
	$result = null;
	
	echo "HTTP result code: ".$code."<br>\n";
	if ($code==200) {
		$result = json_decode($body, true);
		foreach ($result["nzbs"] as $nzb) {
			echo "Found NZB: ".$nzb["uiTitle"]."<br>\n";
		}
	}
	curl_close($ch);
	
	return $result;
}

//======================================
//     Application entry point
//======================================

//A. get server nonce for login
//B. generate a client nonce (cnonce)
//C. login using api-key, both nonces
//D. Add a new download & list queue
$nonce = getNonce();
if ($nonce!=null) {
	$cnonce = gen_uuid();
	$sessionID = login($nonce, $cnonce);
	if ($sessionID!=null) {
	
		//Add an NZB via the API
		//$nzb_url="http://nzbindex.nl/download/79721906/The.Lord.Of.The.Rings.The.Return.Of.The.King.2003.DVD9.TR.EN.Untouched.LecterDisc.par2-846-GB.nzb";
		//addNZB($nzb_url, $sessionID);
		
		//List Queue
		listNZBs($sessionID);
	} else {
		echo "Failed to login, check api-key";
	}
} else {
	echo "Failed to get nonce, check server address";
}
?>
